import { BehaviorSubject, combineLatest } from 'rxjs';
import { distinctUntilChanged, map } from 'rxjs/operators';

// Based on "SvelteSubject" https://github.com/ReactiveX/rxjs/issues/4740#issuecomment-490601347
class Writable extends BehaviorSubject {
    constructor(initialValue) {
        super(initialValue);
    }
    set(value) {
        if (value !== this.getValue()) {
            super.next(value);
        }
    }
    update(fn) {
        this.set(fn(this.getValue()));
    }
    lift(operator) {
        const result = new Writable(operator);
        result.operator = operator;
        result.source = this;
        return result;
    }
}
function readable(initialState) {
    // Observable is our Readable!
    const stateSource = new Writable(initialState);
    return stateSource.asObservable();
}
function writable(initialState) {
    const stateSource = new Writable(initialState);
    return stateSource;
}
function derived(stores, fn, initial_value // TODO
) {
    const single = !Array.isArray(stores);
    const storesArray = single
        ? [stores]
        : stores;
    return combineLatest(storesArray).pipe(distinctUntilChanged((prev, curr) => prev.every((prevItem, i) => prevItem === curr[i])), map(values => fn(single ? values[0] : values)));
}

/*
 * Public API Surface of svelte-store-rxjs
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Writable, derived, readable, writable };
//# sourceMappingURL=svelte-store-rxjs.js.map
