import { __extends } from 'tslib';
import { BehaviorSubject, combineLatest } from 'rxjs';
import { distinctUntilChanged, map } from 'rxjs/operators';

// Based on "SvelteSubject" https://github.com/ReactiveX/rxjs/issues/4740#issuecomment-490601347
var Writable = /** @class */ (function (_super) {
    __extends(Writable, _super);
    function Writable(initialValue) {
        return _super.call(this, initialValue) || this;
    }
    Writable.prototype.set = function (value) {
        if (value !== this.getValue()) {
            _super.prototype.next.call(this, value);
        }
    };
    Writable.prototype.update = function (fn) {
        this.set(fn(this.getValue()));
    };
    Writable.prototype.lift = function (operator) {
        var result = new Writable(operator);
        result.operator = operator;
        result.source = this;
        return result;
    };
    return Writable;
}(BehaviorSubject));
function readable(initialState) {
    // Observable is our Readable!
    var stateSource = new Writable(initialState);
    return stateSource.asObservable();
}
function writable(initialState) {
    var stateSource = new Writable(initialState);
    return stateSource;
}
function derived(stores, fn, initial_value // TODO
) {
    var single = !Array.isArray(stores);
    var storesArray = single
        ? [stores]
        : stores;
    return combineLatest(storesArray).pipe(distinctUntilChanged(function (prev, curr) {
        return prev.every(function (prevItem, i) { return prevItem === curr[i]; });
    }), map(function (values) { return fn(single ? values[0] : values); }));
}

/*
 * Public API Surface of svelte-store-rxjs
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Writable, derived, readable, writable };
//# sourceMappingURL=svelte-store-rxjs.js.map
